from direct.showbase.DirectObject import DirectObject
import math
from panda3d.core import *


class Player (DirectObject) :
    
    def __init__(self) :
        
        self.accept("arrow_up", self.actions, ["forward", 1])
        self.accept("arrow_up-up", self.actions, ["forward", 0])
        self.accept("arrow_down", self.actions, ["backward", 1])
        self.accept("arrow_down-up", self.actions, ["backward", 0])
        self.accept("arrow_left", self.actions, ["left", 1])
        self.accept("arrow_left-up", self.actions, ["left", 0])
        self.accept("arrow_right", self.actions, ["right", 1])
        self.accept("arrow_right-up", self.actions, ["right", 0])
        
        self.accept("space", self.actions, ["jump", 1])
        self.accept("space-up", self.actions, ["jump", 0])
        
        self.keymap = {"forward":0, "left":0, "right":0, "backward":0, "jump":0}

        self.walkspeed = 6
        self.jumpspeed = 8
        
        self.body = loader.loadModel('cube')
        self.body.setPos(24,24,30)
        self.body.setH(0)
        self.body.reparentTo(render)
        
        self.head = loader.loadModel("cube")
        self.head = self.body.attachNewNode("head")
        self.head.setPos(0,0,2)
                
        self.m360 = Mat4(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0)
        self.m45 = Mat4(0.7, 0.7, 0, 0, -0.7, 0.7, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1)
        self.m90 = Mat4(0, 1, 0, 0, -1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,1)
        self.m135 = Mat4(-0.7, 0.7, 0, 0, -0.7, -0.7, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1)
        self.m180 = Mat4(-1, 0, 0, 0, 0, -1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1)
        self.m225 = Mat4(-0.7, -0.7, 0, 0, 0.7, -0.7, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1)
        self.m270 = Mat4(0, -1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1)
        self.m315 = Mat4(0.7, -0.7, 0, 0, 0.7, 0.7, 0, 0, 0 ,0 ,1, 0, 0, 0, 0, 1)
        self.midentity = Mat4(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1)
        
        taskMgr.add(self.turnHead, "turnHead")

    def getPos(self) :
        
        return self.body.getPos()
        
    def setPos(self, (x, y, z)) :
        
        self.body.setPos(x, y, z)
        
    def getVec(self) :
        
        mat = self.body.getNetTransform().getMat() 
        
        if self.keymap["forward"] == 1 and self.keymap["right"] == 1 :
            mat = mat * self.m315
        elif self.keymap["forward"] == 1 and self.keymap["left"] == 1 :
            mat = mat * self.m45
        elif self.keymap["forward"] == 1 :
            mat = mat
        elif self.keymap["backward"] == 1 and self.keymap["left"] == 1: 
            mat = mat * self.m135
        elif self.keymap["backward"] == 1 and self.keymap["right"] == 1: 
            mat = mat * self.m225
        elif self.keymap["backward"] == 1 : 
            mat = mat * self.m180
        elif self.keymap["left"] == 1 :
            mat = mat * self.m90
        elif self.keymap["right"] == 1:
            mat = mat * self.m270
        else :        
            mat = self.midentity
        
        return mat.getRow3(1)
        
    
        

    def turnHead(self, Task) :
    
        mpos = base.win.getPointer(0)

        if base.win.movePointer(0, base.win.getXSize()/2, base.win.getYSize()/2):

            p = (self.head.getP() - (mpos.getY() - base.win.getYSize()/2)*0.1)
            h = (self.body.getH() -  (mpos.getX() - base.win.getXSize()/2)*0.1)

            if p < -90 :
                p = -90
            elif p > 90 :
                p = 90

            self.head.setP(p)
            self.body.setH(h)

        return Task.cont
        
        
    def actions(self, action, etat) :

        self.keymap[action] = etat
        
