from pandac.PandaModules import *


class Geomizer() :
    
    def __init__(self) :
        
        self.format = GeomVertexFormat.getV3n3c4t2()
        self.vdata = GeomVertexData('vertices', self.format, Geom.UHDynamic)
        self.geom = Geom(self.vdata)
        self.vertex = GeomVertexWriter(self.vdata, 'vertex')
        self.color = GeomVertexWriter(self.vdata, 'color')
        self.texcoord = GeomVertexWriter(self.vdata, 'texcoord')
        self.normal = GeomVertexWriter(self.vdata, 'normal')        
        self.fnormals = [ (1,0,0), (-1,0,0), (0,1,0), (0,-1,0), (0,0,1), (0,0,-1) ]        
        self.numvertex = -1        
  
    def addCube ( self, todraw, vertices, uvs) :
        
        for f in range (6) :
            if todraw[f] == 1 :        
                for v in range(4) :
                    self.vertex.addData3f(vertices[f][v])
                    self.texcoord.addData2f(uvs[v])
                    self.normal.addData3f(self.fnormals[f])
                    self.numvertex += 1        
                prim = GeomTristrips(Geom.UHStatic)
                prim.addVertices(self.numvertex-3, self.numvertex-2, self.numvertex-1, self.numvertex)
                self.geom.addPrimitive(prim)
        
    def getGeom(self) :
        return self.geom






class Renderer() :
    
    def __init__(self) : 
        self.t = 0
        self.g = Geomizer()
        
    def addFace(self, face, vertices) :        
        self.g.addFace(face, vertices)
        
    def addCube(self, todraw, vertices, value) :        
        uvs = self.getUV(value)
        texture = self.getTexture()
        self.g.addCube(todraw, vertices, uvs)
        
    def getGeom(self) :
        return self.g.getGeom()
        
    def setTexPack(self, texpack) :
        self.t = texpack
        
    def setGeomizer(self, geomizer) :        
        self.g = geomizer
        
    def getUV(self, voxvalue) :        
        uvtable = [(0,0), (1,1), (2,1), (3,1), (4,1)]
        coords = uvtable[voxvalue]
        return self.t.getUV(coords)
        
    def getTexture(self) :        
        return self.t.texture





class TexPack() :
    
    def __init__(self,  texfile,  col,  row =1):        
        self.texture = loader.loadTexture(texfile)
        self.xunit = float(1.0/col)
        self.yunit = float (1.0/row)
        
    def getUV(self,  coords):
        
        mx = (coords[0]-1)*self.xunit
        px = coords[0]*self.xunit
        my = (coords[1]-1)*self.yunit
        py = coords[1]*self.yunit
        
        if mx <0 : mx = 0
        if px >1 : px = 1
        if my<0 : my = 0
        if py>1 : py = 1
        uv = [    (px, py),  (mx, py)  ,   (px, my) ,  (mx, my)  ]
       
        return uv
